
from django.contrib import admin

from modeltranslation.admin import TranslationAdmin

from articles.models import Article, ArticleTag, ArticleType


class ArticleAdmin(TranslationAdmin):

    list_display = ['title', 'type', 'created', 'is_comments_enabled']

    list_filter = ['type', 'created', 'is_comments_enabled']

    filter_horizontal = ['tags']


class ArticleTagAdmin(TranslationAdmin):

    list_display = ['text']
    search_fields = ['text']


class ArticleTypeAdmin(TranslationAdmin):

    list_display = ['name']
    search_fields = ['name']


admin.site.register(Article, ArticleAdmin)
admin.site.register(ArticleTag, ArticleTagAdmin)
admin.site.register(ArticleType, ArticleTypeAdmin)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.4 on 2018-01-10 12:10
from __future__ import unicode_literals

import ckeditor_uploader.fields
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    operations = [
        migrations.CreateModel(
            name='Article',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(max_length=255, verbose_name='Title')),
                ('logo', models.ImageField(blank=True, null=True, upload_to=b'article_logos')),
                ('description', models.CharField(max_length=255, verbose_name='Description')),
                ('text', ckeditor_uploader.fields.RichTextUploadingField(max_length=50000, verbose_name='Text')),
                ('created', models.DateTimeField(db_index=True, verbose_name='Created')),
                ('author', models.CharField(blank=True, max_length=255, verbose_name='Author')),
                ('is_comments_enabled', models.BooleanField(default=True, verbose_name='Is comments enabled')),
            ],
            options={
                'ordering': ['-created'],
                'verbose_name': 'Article',
                'verbose_name_plural': 'Articles',
            },
        ),
        migrations.CreateModel(
            name='ArticleTag',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('text', models.CharField(max_length=255, unique=True, verbose_name='Text')),
            ],
            options={
                'verbose_name': 'Article tag',
                'verbose_name_plural': 'Article tags',
            },
        ),
        migrations.CreateModel(
            name='ArticleType',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255, unique=True, verbose_name='Name')),
                ('slug', models.CharField(db_index=True, max_length=255, unique=True, verbose_name='Slug')),
            ],
            options={
                'verbose_name': 'Article type',
                'verbose_name_plural': 'Article types',
            },
        ),
        migrations.AddField(
            model_name='article',
            name='tags',
            field=models.ManyToManyField(blank=True, related_name='tags', to='articles.ArticleTag', verbose_name='Tags'),
        ),
        migrations.AddField(
            model_name='article',
            name='type',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='articles', to='articles.ArticleType', verbose_name='Type'),
        ),
    ]




from django.shortcuts import get_object_or_404
from django.views.generic import ListView

from pure_pagination import PaginationMixin
from hitcount.views import HitCountDetailView

from articles.models import Article, ArticleType


class ArticleListView(PaginationMixin, ListView):

    template_name = 'articles/list.html'
    paginate_by = 10

    def get(self, request, *args, **kwargs):
        self.article_type = get_object_or_404(ArticleType, slug=kwargs['type'])
        return super(ArticleListView, self).get(request, *args, **kwargs)

    def get_queryset(self):
        return self.article_type.articles.all()

    def get_context_data(self, **kwargs):
        cxt = super(ArticleListView, self).get_context_data(**kwargs)
        cxt['article_type'] = self.article_type
        return cxt


class ArticleDetailView(HitCountDetailView):

    template_name = 'articles/info.html'
    count_hit = True

    def get_object(self, queryset=None):
        return get_object_or_404(
            Article,
            type__slug=self.kwargs['type'],
            pk=self.kwargs['id'])




from modeltranslation.translator import translator

from articles.models import Article, ArticleTag, ArticleType


translator.register(Article, fields=['title', 'description', 'text'])
translator.register(ArticleTag, fields=['text'])
translator.register(ArticleType, fields=['name'])




from django.urls import path

from articles import views


app_name = 'articles'


urlpatterns = [

    path('<str:type>/', views.ArticleListView.as_view(), name='list'),

    path('<str:type>/<str:slug>_<int:id>/',
         views.ArticleDetailView.as_view(), name='info')

]




from random import randint

from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.urls import reverse_lazy
from django.contrib.contenttypes.models import ContentType
from django.utils.functional import cached_property

from ckeditor_uploader.fields import RichTextUploadingField
from slugify import slugify_url
from hitcount.models import HitCount


class ArticleType(models.Model):

    name = models.CharField(_('Name'), max_length=255, unique=True)

    slug = models.CharField(
        _('Slug'), max_length=255, db_index=True, unique=True)

    def __str__(self):
        return self.name

    class Meta:
        verbose_name = _('Article type')
        verbose_name_plural = _('Article types')


class ArticleTag(models.Model):

    text = models.CharField(_('Text'), max_length=255, unique=True)

    def __str__(self):
        return self.text

    class Meta:
        verbose_name = _('Article tag')
        verbose_name_plural = _('Article tags')


class Article(models.Model):

    type = models.ForeignKey(
        ArticleType, verbose_name=_('Type'), blank=True, null=True,
        related_name='articles', on_delete=models.SET_NULL)

    title = models.CharField(_('Title'), max_length=255)

    logo = models.ImageField(
        blank=True, null=True, upload_to='article_logos')

    description = models.CharField(_('Description'), max_length=255)

    text = RichTextUploadingField(_('Text'), max_length=50000)

    created = models.DateTimeField(_('Created'), db_index=True)

    author = models.CharField(_('Author'), max_length=255, blank=True)

    is_comments_enabled = models.BooleanField(
        _('Is comments enabled'), default=True)

    tags = models.ManyToManyField(
        ArticleTag, verbose_name=_("Tags"), related_name='tags', blank=True)

    @classmethod
    def most_popular(cls, article_type):

        ct = ContentType.objects.get_for_model(cls)

        ids = HitCount.objects.filter(content_type=ct).order_by('-hits')\
            .values_list('object_pk', flat=True)

        return cls.objects.filter(
            type__slug=article_type,
            id__in=ids
        ).extra(
            select={
                'manual': 'FIELD(articles_article.id,%s)' %
                          ','.join(map(str, ids))
            },
            order_by=['manual']
        )

    @classmethod
    def get_related_articles(cls, article_type, exclude_pk, count=6):

        index = 0

        related_articles = cls.objects.filter(
            type__slug=article_type
        ).exclude(
            pk=exclude_pk
        )

        related_articles_count = len(related_articles)

        if related_articles_count:

            if related_articles_count > count:
                index = randint(0, related_articles_count - count)

            return related_articles[index:index + count]

        return []

    @cached_property
    def related_articles(self):
        return self.get_related_articles(self.type.slug, self.pk)

    @property
    def hits(self):
        return HitCount.objects.get_for_object(self).hits

    @property
    def slug(self):
        return slugify_url(self.title, separator='_')

    def __str__(self):
        return self.title

    def get_absolute_url(self):
        return reverse_lazy('articles:info', kwargs={
            'type': self.type.slug,
            'slug': self.slug,
            'id': self.pk
        })

    class Meta:
        verbose_name = _('Article')
        verbose_name_plural = _('Articles')
        ordering = ['-created']
